<?php
namespace Controller;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Util\Instance\ServiceInstance;
use Util\Utilities;
class Table {

    public function findTable(DTO\Request\Filter $request, DTO\Response $response)
    {
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $filters = $request->getFilterCriteria()->getConditions();
        $tables = ServiceInstance::getTable()->findTable($filters, $page, $pageSize);
        $response->addDataSet(new DataSource\DataSet('table', $tables));
    }

    public function findTableObservations(DTO\Request\Filter $request, DTO\Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $table = Utilities::getValueInFilterDataSourceByName($filters, 'TABLE', true);
        $tables = ServiceInstance::getTable()->findTableObservations($table);
        $response->addDataSet(new DataSource\DataSet('findTableObservations', [$tables]));
    }

    public function findtableColumnObservations(DTO\Request\Filter $request, DTO\Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $column = Utilities::getValueInFilterDataSourceByName($filters, 'COLUMN', true);
        $table  = Utilities::getValueInFilterDataSourceByName($filters, 'TABLE', true);
        $columnObservation  = ServiceInstance::getTable()->findtableColumnObservations($column, $table);
        $response->addDataSet(new DataSource\DataSet('findTableColumnObservations', [$columnObservation]));
    }

    public function findTableColumns(DTO\Request\Filter $request, DTO\Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $primaryKey = Utilities::getValueInFilterDataSourceByName($filters, 'PK', true);
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $columns = ServiceInstance::getTable()->findTableColumns($filters, $primaryKey , $page, $pageSize);
        $response->addDataSet(new DataSource\DataSet('tableColumns', $columns));
    }

    public function findTableConstraints(DTO\Request\Filter $request, DTO\Response $response){
        $filters = $request->getFilterCriteria()->getConditions();
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $constrains = ServiceInstance::getTable()->findTableConstraints($filters, $page, $pageSize);
        $response->addDataSet(new DataSource\DataSet('tableConstrains', $constrains));
    }

    public function findTableTrigger(DTO\Request\Filter $request, DTO\Response $response){
        $filters = $request->getFilterCriteria()->getConditions();
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $triggers = ServiceInstance::getTable()->findTableTrigger($filters, $page, $pageSize);
        $response->addDataSet(new DataSource\DataSet('tableTrigger', $triggers));
    }

    public function findTableConstraintColumns(DTO\Request\Filter $request, DTO\Response $response){
        $filters = $request->getFilterCriteria()->getConditions();
        $page = $request->getFilterCriteria()->getPage();
        $constraintIsFk = Utilities::getValueInFilterDataSourceByName($filters, 'FK', true);
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $constraintCollumns = ServiceInstance::getTable()->findTableConstraintColumns($filters, $page, $pageSize, $constraintIsFk);
        $response->addDataSet(new DataSource\DataSet('tableTrigger', $constraintCollumns));
    }

    public function saveTableObservation(DTO\Request\Dataset $request, DTO\Response $response)
    {

        try {
            $row = $request->getDataSet()->getRows()[0];
            $currentRow = $row["row"];
            $table = $currentRow["TABLE_NAME"];
            $observation = $currentRow["OBSERVATION"];
            $result = ServiceInstance::getTable()->saveTableObservations($table, $observation);
            $response->addDataSet(new DataSource\DataSet('tableObservations', [$result]));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function saveTableColumnObservation(DTO\Request\Dataset $request, DTO\Response $response)
    {

        try {
            $row = $request->getDataSet()->getRows()[0];
            $currentRow = $row["row"];
            $column = $currentRow["COLUMN_NAME"];
            $table  = $currentRow["TABLE_NAME"];
            $observation = $currentRow["OBSERVATION"];
            $result = ServiceInstance::getTable()->saveTableColumnObservation($table, $column, $observation);
            $response->addDataSet(new DataSource\DataSet('tableColumnObservations', [$result]));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
}